package com.scbfkj.uni.api;

import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.service.SecurityService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.Map;

@Component
@Aspect
public class LogAop {

    @Resource
    private SecurityService securityService;


    @Around(value = "within( com.scbfkj.uni.api.*)")
    public Object invokeAround(ProceedingJoinPoint joinPoint) {
//        当前时间戳
        long beforeTime = System.currentTimeMillis();
        try {

//        请求
            ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//
            HttpServletRequest request = requestAttributes.getRequest();
            String uri = request.getRequestURI();
//        请求参数
            Object[] args = joinPoint.getArgs();
            switch (uri) {
//                检查应用令牌
                case "/user/verifyCode", "/foxlibc/verification-code", "/user/forceLogin", "/foxlibc/force_sign", "/user/refreshToken" -> {
                    String appToken = request.getHeader("token");
                    Map<String, Object> result = securityService.verifyToken(appToken);
                    if (!result.get("code").equals("0")) {
                        return result;
                    }
                }
//                不检查，直接放行
                case "/user/getToken" -> {
                }
//                检查用户令牌
                default -> {
                    String userToken = request.getHeader("usertoken");
                    Map<String, Object> result = securityService.checkUserToken(userToken);
                    if (!result.get("code").equals("0")) {
                        return result;
                    }
                }
            }
            return joinPoint.proceed(args);
        } catch (Throwable e) {
            e.printStackTrace();
//            错误异常消息
            return ResponseEntity.ok(UniReturnUtil.fail(e.getMessage()));
        } finally {
            //        执行后时间戳
            long afterTime = System.currentTimeMillis();
//        执行时间
            long execTime = afterTime - beforeTime;
            System.out.println("执行时长：" + execTime);
//            todo 写日志
        }
    }
}
