package com.scbfkj.uni.system;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.oracle.truffle.api.Option;
import com.scbfkj.uni.library.DataEncryptionUtil;
import com.scbfkj.uni.library.DataFormatUtil;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.pool.HikariPool;
import jakarta.annotation.Nonnull;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

@Component
public class Config {

    public static String centerConnectionStr;
    public static String securityConnectionStr;


    private final static String TYPE = "RSA";

    private final static String ALGORITHM = "RSA/ECB/PKCS1Padding";

    private final static Map<String, HikariPool> dataSourcePools = new HashMap<>();


    public static HikariPool getDataSourcePool(String connectionStr) throws Exception {
        if (dataSourcePools.containsKey(connectionStr)) {
            return dataSourcePools.get(connectionStr);
        }
        JsonNode jsonNode = DataFormatUtil.stringToJsonNode(connectionStr);

        JsonNode jdbcUrl = jsonNode.get("jdbcUrl");
        JsonNode username = jsonNode.get("username");
        JsonNode password = jsonNode.get("password");
        JsonNode driverClassName = jsonNode.get("driverClassName");
        HikariPool dataSourcePool = createDataSourcePool(jdbcUrl.asText(), Objects.isNull(username) ? null : username.asText(), Objects.isNull(password) ? null : password.asText(), driverClassName.asText(), null);
        dataSourcePools.put(connectionStr, dataSourcePool);
        return dataSourcePool;
    }


    public static HikariPool createDataSourcePool(@Nonnull String url, String username, String password, String driver, String poolName) throws Exception {
        HikariConfig hikariConfig = new HikariConfig();
        hikariConfig.setMaximumPoolSize(Math.min(Runtime.getRuntime().availableProcessors(), 8));//最大核心数:当前可用CPU数，最大16
        hikariConfig.setKeepaliveTime(60000); //用于跟数据库保持心跳连接
        hikariConfig.setMaxLifetime(60000 * 60 * 4); //4小时 接在池中的最大生存时间，超过该时间强制逐出
        hikariConfig.addDataSourceProperty("cachePrepStmts", "true"); //是否自定义配置，为true时下面两个参数才生效
        hikariConfig.addDataSourceProperty("prepStmtCacheSize", "250"); //连接池大小默认25，官方推荐250-500
        hikariConfig.addDataSourceProperty("prepStmtCacheSqlLimit", "2048"); //单条语句最大长度默认256，官方推荐2048
        hikariConfig.addDataSourceProperty("useServerPrepStmts", "true"); //新版本MySQL支持服务器端准备，开启能够得到显著性能提升
        hikariConfig.addDataSourceProperty("useLocalSessionState", "true");
        hikariConfig.addDataSourceProperty("useLocalTransactionState", "true");
        hikariConfig.addDataSourceProperty("rewriteBatchedStatements", "true");//批量新增
        hikariConfig.addDataSourceProperty("cacheResultSetMetadata", "true");
        hikariConfig.addDataSourceProperty("cacheServerConfiguration", "true");
        hikariConfig.addDataSourceProperty("elideSetAutoCommits", "true");
        hikariConfig.addDataSourceProperty("maintainTimeStats", "false");
        String privateKeyStr = SpringContextApplication.getString("cer.private-key");

        hikariConfig.setJdbcUrl(DataEncryptionUtil.decryptByPrivateKey(url, privateKeyStr, TYPE, ALGORITHM));

        if (Objects.nonNull(username)) {
            hikariConfig.setUsername(DataEncryptionUtil.decryptByPrivateKey(username, privateKeyStr, TYPE, ALGORITHM));
        }
        if (Objects.nonNull(password)) {
            hikariConfig.setPassword(DataEncryptionUtil.decryptByPrivateKey(password, privateKeyStr, TYPE, ALGORITHM));
        }
        if (Objects.nonNull(driver)) {
            hikariConfig.setDriverClassName(driver);
        }
        if (Objects.nonNull(poolName)) {
            hikariConfig.setPoolName(poolName);
        }
        hikariConfig.setAutoCommit(false);
        return new HikariPool(hikariConfig);
    }
}
